library(car)
library(caret)
library(class)
library(dplyr)
library(ggplot2)
library(tidyr)
library(tidyverse)
library(performance)
library(corrplot)
library(leaps)
library(glancedata)
library(broom)
library(modelr)
library(prediction)
library(caret)

dia<- read.csv("diabetes.csv")
dia
str(dia)

#change the outcome according to diabetes and not diabetes
diab <- dia %>% mutate(Outcome = factor(Outcome, levels = c(0,1), labels = c("Not Diabetes", "Diabetes")))
diab
glimpse(diab)

#Checking of missing values in the dataset
colSums(is.na(dia))

#checking for null values in the dataset
is.null(dia)

#checking for duplication
uni<-unique(diab)
dim(uni)[1]

#Check the target variable
x <- prop.table(table(diab$Outcome))
b <- barplot(x,col="lightBlue", main = "Target Class Proportion Diagram")
text(x=b, y= x, labels=round(x,2), pos = 1)

#checking of outliers for each variables and comparing with the outcome
 
# 2. Pregnancies
p1 <- ggplot(diab, aes(x = Outcome, y = Pregnancies,fill = Outcome)) +
  geom_boxplot() +
  theme(legend.position = "bottom") +
  ggtitle("Number of pregnancies Vs Diabetes")

p2 <- ggplot(diab,aes(x = Pregnancies,fill = factor(Outcome))) + 
  geom_bar(position = "Dodge") + 
  scale_x_continuous(limits = c(0,16)) +
  theme(legend.position = "bottom") +
  labs(title = "Pregnancies Vs Outcome")

gridExtra::grid.arrange(p1, p2, ncol = 2)

# 2. glucouse
p2 <- ggplot(diab, aes(x = Glucose, color = Outcome, fill = Outcome)) +
  geom_density(alpha = 0.8) +
  theme(legend.position = "bottom") +
  labs(x = "Glucose", y = "Density", title = "Density plot of glucose")

p1 <- ggplot(diab, aes(x = Outcome, y = Glucose,fill = Outcome)) +
  geom_boxplot() +
  theme(legend.position = "bottom") +
  ggtitle("Variation of glucose in women Vs Diabetes")

gridExtra::grid.arrange(p1, p2, ncol = 2)

# 3. Blood Pressure
p2 <- ggplot(diab, aes(x = BloodPressure, color = Outcome, fill = Outcome)) +
  geom_density(alpha = 0.8) +
  theme(legend.position = "bottom") +
  labs(x = "Blood pressure", y = "Density", title = "Density plot of Blood pressure")

p1 <- ggplot(diab, aes(x = Outcome, y = BloodPressure,fill = Outcome)) +
  geom_boxplot() +
  theme(legend.position = "bottom") +
  ggtitle("Variation of blood pressure in women Vs Diabetes")

gridExtra::grid.arrange(p1, p2, ncol = 2)

# 4. Skin Thickness
p2 <- ggplot(diab, aes(x = SkinThickness, color = Outcome, fill = Outcome)) +
  geom_density(alpha = 0.8) +
  theme(legend.position = "bottom") +
  labs(x = "Skin thickness", y = "Density", title = "Density plot of skin thickness")

p1 <- ggplot(diab, aes(x = Outcome, y = SkinThickness,fill = Outcome)) +
  geom_boxplot() +
  theme(legend.position = "bottom") +
  ggtitle("Variation of skin thickness Vs Diabetes")

gridExtra::grid.arrange(p1, p2, ncol = 2)

# 5. Insulin
p1 <- ggplot(diab, aes(x = Outcome, y = Insulin,fill = Outcome)) +
  geom_boxplot() + 
  theme(legend.position = "bottom") +
  ggtitle("Variation of Insulin content Vs Diabetes")

p2 <- ggplot(diab, aes(Insulin, fill = Outcome)) +
  geom_histogram(binwidth=10) +
  theme(legend.position = "bottom") +
  ggtitle("Variation of Insulin content Vs Diabetes")

gridExtra::grid.arrange(p1, p2, ncol = 2)

# 6. BMI
p2 <- ggplot(diab, aes(BMI, fill = Outcome)) +
  geom_histogram() +
  theme(legend.position = "bottom") +
  ggtitle("Variation of BMI of women Vs Diabetes")

p1 <- ggplot(diab, aes(x = Outcome, y = BMI,fill = Outcome)) +
  geom_boxplot(binwidth = 5) +
  theme(legend.position = "bottom") +
  ggtitle("Variation of BMI of women Vs Diabetes")

gridExtra::grid.arrange(p1, p2, ncol = 2)

# 7. DPF
p1 <- ggplot(diab, aes(x = Outcome, y = DiabetesPedigreeFunction,fill = Outcome)) +
  geom_boxplot() +
  theme(legend.position = "bottom") +
  ggtitle("Variation of DPF of women Vs Diabetes")


p2 <- ggplot(diab, aes(DiabetesPedigreeFunction,fill = Outcome)) +
  geom_histogram() +
  theme(legend.position = "bottom") +
  ggtitle("Variation of DPF Vs Diabetes")

gridExtra::grid.arrange(p1, p2, ncol = 2)

# 8. Age
p2 <- ggplot(diab, aes(Age, fill = Outcome)) +
  geom_histogram(binwidth = 5) +
  theme(legend.position = "bottom") +
  ggtitle("Variation of Age of women Vs Diabetes")

p1 <- ggplot(diab, aes(x = Outcome, y = Age,fill = Outcome)) +
  geom_boxplot() +
  theme(legend.position = "bottom") +
  ggtitle("Variation of Age of women Vs Diabetes")

gridExtra::grid.arrange(p1, p2, ncol = 2)


sum(diab$BloodPressure==0)
sum(diab$Glucose==0)
sum(diab$SkinThickness==0)
sum(diab$Insulin==0)
sum(diab$BMI==0)

# People with no diabetes an zero blood pressure values
non_diabetic <- which(diab$BloodPressure==0 & diab$Outcome==0)
temp_median <- median(diab$BloodPressure[!diab$BloodPressure==0 & diab$Outcome==0])
diab$BloodPressure[non_diabetic] <- temp_median

# People with diabetes an zero blood pressure values
diabetic <- which(diab$BloodPressure==0 & diab$Outcome==1)
diab$BloodPressure[diabetic] <- median(diab$BloodPressure[!diab$BloodPressure==0 & diab$Outcome==1])

# People with no diabetes an zero Glucose values
non_diabetic <- which(diab$Glucose==0 & diab$Outcome==0)
diab$Glucose[non_diabetic] <- median(diab$Glucose[!diab$Glucose==0 & diab$Outcome==0])

# People with diabetes an zero Glucose values
diabetic <- which(diab$Glucose==0 & diab$Outcome==1)
diab$Glucose[diabetic] <- median(diab$Glucose[!diab$Glucose==0 & diab$Outcome==1])

# People with no diabetes an zero Skin Thickness values
non_diabetic <- which(diab$SkinThickness==0 & diab$Outcome==0)
diab$SkinThickness[non_diabetic] <- median(diab$SkinThickness[!diab$SkinThickness==0 & diab$Outcome==0])

# People with diabetes an zero Skin Thickness values
diabetic <- which(diab$SkinThickness==0 & diab$Outcome==1)
diab$SkinThickness[diabetic] <- median(diab$SkinThickness[!diab$SkinThickness==0 & diab$Outcome==1])

# People with no diabetes an zero Insulin values
non_diabetic <- which(diab$Insulin==0 & diab$Outcome==0)
diab$Insulin[non_diabetic] <- median(diab$Insulin[!diab$Insulin==0 & diab$Outcome==0])

# People with diabetes an zero Insulin values
diabetic <- which(diab$Insulin==0 & diab$Outcome==1)
diab$Insulin[diabetic] <- median(diab$Insulin[!diab$Insulin==0 & diab$Outcome==1])

# People with no diabetes an zero BMI values
non_diabetic <- which(diab$BMI==0 & diab$Outcome==0)
diab$BMI[non_diabetic] <- median(diab$BMI[!diab$BMI==0 & diab$Outcome==0])

# People with diabetes an zero BMI values
diabetic <- which(diab$BMI==0 & diab$Outcome==1)
diab$BMI[diabetic] <- median(diab$BMI[!diab$BMI==0 & diab$Outcome==1])

summary(diab)


#correlation bewteen the variables
corMat = cor (diab[, -9])
diag (corMat) = 0 #Remove self correlations
corrplot.mixed(corMat,tl.pos = "lt")

#Variable selection
#Building models with all the variables

model = glm(Outcome ~ .,data = diab,family="binomial")
summary(model)
glance(model)
glance(model) %>%
  dplyr::select(adj.r.squared, sigma, AIC, BIC, p.value)

#Model based on p-value

model1 = glm(Outcome ~ .-BloodPressure-Age,diab,family="binomial")
summary(model1)
subset_result <- regsubsets(Outcome~.,diab, nbest=2, nvmax = 14)
summary(subset_result)
plot(subset_result, scale="bic")

#Model based on BIC 
model2 = glm(Outcome~.-BloodPressure-SkinThickness-Age,data=diab,family="binomial")
summary(model2)


#Model based on AIC
model3 = glm(Outcome ~ Pregnancies + Glucose + SkinThickness + Insulin + BMI + 
               DiabetesPedigreeFunction,diab,family="binomial")
summary(model3)

#Final model 
final = glm(Outcome ~ .-BloodPressure-Age,diab,family="binomial")
summary(final)

#Model validation
require(caTools)
set.seed(3)
sample = sample.split(diab$Outcome, SplitRatio=0.75)
train = subset(diab, sample==TRUE)
test = subset(diab, sample==FALSE)
nrow(diab)
nrow(train)
nrow(test)
table(diab$Outcome)

#Training all the models
pre <- glm(Outcome ~ ., data = train, family = binomial)
summary(pre)

#Training the choosen final model
pre1 <- glm(Outcome ~ . -Age-BloodPressure, data = train, family = binomial)
summary(pre1)

# Let's predict outcome on Training dataset

PredictTrain <- predict(pre1, type = "response")
summary(PredictTrain)

# This computes the average prediction for each of the two outcomes

tapply(PredictTrain, train$Outcome, mean)

# Build confusion matrix with a threshold value of 0.7

threshold_0.7 <- table(train$Outcome, PredictTrain > 0.7)
threshold_0.7

# Accuracy
accuracy_0.7 <- round(sum(diag(threshold_0.7))/sum(threshold_0.7),2)
sprintf('Accuracy is %s', accuracy_0.7)

# Mis-classification error rate
MC_0.7 <- 1-accuracy_0.7
sprintf("Mis-classification error is %s",MC_0.7)

sensitivity0.7 <- round(78/(123+78),2)
specificity0.7 <- round(359/(359+16),2)
sprintf("Sensitivity at 0.7 threshold: %s", sensitivity0.7)
sprintf("Specificity at 0.7 threshold: %s", specificity0.7)

# Generate ROC Curves

library(ROCR)

ROCRpred = prediction(PredictTrain, train$Outcome)
ROCRperf = performance(ROCRpred, "tpr", "fpr")

# Adding threshold labels
plot(ROCRperf, colorize=TRUE, print.cutoffs.at = seq(0,1,0.1), text.adj = c(-0.2, 1.7))
abline(a=0, b=1)

auc_train <- round(as.numeric(performance(ROCRpred, "auc")@y.values),2)
legend(.8, .2, auc_train, title = "AUC", cex=1)

#Testing the data
PredictTest <- predict(pre1, type = "response", newdata = test)

# Convert probabilities to values using the below

## Based on ROC curve above, selected a threshold of 0.5
test_tab <- table(test$Outcome, PredictTest > 0.5)
test_tab

accuracy_test <- round(sum(diag(test_tab))/sum(test_tab),2)
sprintf("Accuracy on test set is %s", accuracy_test)

# Compute test set AUC

ROCRPredTest = prediction(PredictTest, test$Outcome)
auc = round(as.numeric(performance(ROCRPredTest, "auc")@y.values),2)
auc
